@@ -1,7 +1,7 @@
 from pkgutil import extend_path
-from typing import Iterable
+from typing import List
 
-__path__: Iterable[str] = extend_path(__path__, __name__)
+__path__: List[str] = extend_path(__path__, __name__)
 
 # for BW compat
 from swh.objstorage.factory import *  # noqa
